import { ParameterType } from 'jspsych';

var version = "1.1.0";

const info = {
  name: "spatial-nback",
  version,
  parameters: {
    /** Number of rows in the spatial grid */
    rows: {
      type: ParameterType.INT,
      default: 3
    },
    /** Number of columns in the spatial grid */
    cols: {
      type: ParameterType.INT,
      default: 3
    },
    /** Size of each cell in pixels, this will affect size of whole grid also. If null, defaults to 12vh (12% of viewport height) */
    cell_size: {
      type: ParameterType.INT,
      default: null
    },
    /** Row position of the stimulus (0-indexed). If null, no stimulus is shown. */
    stimulus_row: {
      type: ParameterType.INT,
      default: null
    },
    /** Column position of the stimulus (0-indexed). If null, no stimulus is shown. */
    stimulus_col: {
      type: ParameterType.INT,
      default: null
    },
    /** Whether this trial is a target trial */
    is_target: {
      type: ParameterType.BOOL,
      default: false
    },
    /** Duration the stimulus is displayed (ms). If null, waits indefinitely for response */
    stimulus_duration: {
      type: ParameterType.INT,
      default: null
    },
    /** Inter-stimulus interval (ms) */
    isi_duration: {
      type: ParameterType.INT,
      default: 500
    },
    /** Duration of feedback display (ms) */
    feedback_duration: {
      type: ParameterType.INT,
      default: 0
    },
    /** Whether to show feedback "Incorrect! (231ms)" after response */
    show_feedback_text: {
      type: ParameterType.BOOL,
      default: true
    },
    /** Whether to show feedback border around the grid */
    show_feedback_border: {
      type: ParameterType.BOOL,
      default: true
    },
    /** Labels for the response buttons. First button is for match, second is for no match */
    buttons: {
      type: ParameterType.STRING,
      default: ["MATCH", "NO MATCH"],
      array: true
    },
    /** Index of the match button in the buttons array */
    match_index: {
      type: ParameterType.INT,
      default: 0
    },
    /** Color of the stimulus square */
    stimulus_color: {
      type: ParameterType.STRING,
      default: "#0066cc"
    },
    /** Color of correct feedback border */
    correct_color: {
      type: ParameterType.STRING,
      default: "#00cc00"
    },
    /** Color of incorrect feedback border */
    incorrect_color: {
      type: ParameterType.STRING,
      default: "#cc0000"
    },
    /** Instructions to display above the grid */
    instructions: {
      type: ParameterType.STRING,
      default: ""
    }
  },
  data: {
    /** Row position of the stimulus */
    stimulus_row: {
      type: ParameterType.INT
    },
    /** Column position of the stimulus */
    stimulus_col: {
      type: ParameterType.INT
    },
    /** Whether this trial was a target */
    is_target: {
      type: ParameterType.BOOL
    },
    /** Index of button pressed (match_index for match, other indices for no match), null if no response. */
    response: {
      type: ParameterType.INT
    },
    /** Response time in milliseconds */
    response_time: {
      type: ParameterType.INT
    },
    /** Whether the response was correct, based on match_index parameter and is_target. */
    correct: {
      type: ParameterType.BOOL
    }
  },
  citations: {
    "apa": "",
    "bibtex": ""
  }
};
class SpatialNbackPlugin {
  constructor(jsPsych) {
    this.jsPsych = jsPsych;
  }
  static {
    this.info = info;
  }
  trial(display_element, trial) {
    if (trial.rows <= 1 && trial.cols <= 1) {
      throw new Error("Grid must have more than one cell. Both rows and cols cannot be 1 or less.");
    }
    if (trial.rows <= 0 || trial.cols <= 0) {
      throw new Error(
        "Grid dimensions must be positive integers. Rows and cols must be greater than 0."
      );
    }
    if (trial.match_index >= trial.buttons.length || trial.match_index < 0) {
      throw new Error(`match_index (${trial.match_index}) must be a valid index within the buttons array (length ${trial.buttons.length}).`);
    }
    if (trial.stimulus_row !== null && trial.stimulus_col !== null) {
      if (trial.stimulus_row >= trial.rows || trial.stimulus_col >= trial.cols) {
        throw new Error(
          `Stimulus position (${trial.stimulus_row}, ${trial.stimulus_col}) is outside grid bounds (${trial.rows}x${trial.cols}).`
        );
      }
    }
    let trial_start_time;
    let response_allowed = false;
    let response_given = false;
    let button_pressed = null;
    let stimulus_hidden = false;
    const show_stimulus = trial.stimulus_row !== null && trial.stimulus_col !== null;
    const stimulus_row = show_stimulus ? trial.stimulus_row : null;
    const stimulus_col = show_stimulus ? trial.stimulus_col : null;
    const createDisplay = () => {
      let html = `<div id="nback-container" style="
        display: flex;
        flex-direction: column;
        gap: 5vh;
      ">`;
      html += `<div id="nback-instructions" style="font-size: clamp(15px, 3vmin, 30px);">${trial.instructions}</div>`;
      html += `<div id="nback-grid-container" style="
        display: flex;
        justify-content: center;
        align-items: center;
      ">`;
      const cell_size_style = trial.cell_size !== null ? `${trial.cell_size}px` : "12vh";
      html += `<div id="nback-grid" style="
        border: 3px solid #000;
        box-sizing: border-box;
        display: inline-block;
      ">`;
      for (let row = 0; row < trial.rows; row++) {
        html += '<div style="display: flex;">';
        for (let col = 0; col < trial.cols; col++) {
          html += `<div id="cell-${row}-${col}" style="
            width: ${cell_size_style};
            height: ${cell_size_style};
            border: 1px solid #ccc;
            box-sizing: border-box;
          "></div>`;
        }
        html += "</div>";
      }
      html += "</div></div>";
      html += `<div id="nback-feedback-section" style="
        display: flex;
        flex-direction: column;
        gap: 1.5vmin;
      ">`;
      html += `<div id="nback-feedback" style="visibility: hidden;">Correct! (999ms)</div>`;
      html += `<div id="nback-buttons-container" style="
        display: flex;
        gap: 10px;
        justify-content: center;
      ">`;
      trial.buttons.forEach((buttonText, index) => {
        html += `<button id="nback-response-btn-${index}" class="jspsych-btn nback-response-btn" style="margin: 0px 8px" disabled>${buttonText}</button>`;
      });
      html += `</div>`;
      html += "</div></div>";
      display_element.innerHTML = html;
      trial.buttons.forEach((buttonText, index) => {
        const button = document.getElementById(`nback-response-btn-${index}`);
        button.addEventListener("click", () => handleResponse(index));
      });
    };
    const startTrial = () => {
      if (show_stimulus) {
        const cell = document.getElementById(`cell-${stimulus_row}-${stimulus_col}`);
        cell.style.backgroundColor = trial.stimulus_color;
      }
      response_allowed = true;
      trial_start_time = performance.now();
      stimulus_hidden = false;
      trial.buttons.forEach((buttonText, index) => {
        const responseButton = document.getElementById(`nback-response-btn-${index}`);
        responseButton.disabled = false;
      });
      if (trial.stimulus_duration !== null) {
        this.jsPsych.pluginAPI.setTimeout(() => {
          if (show_stimulus) {
            const cell = document.getElementById(`cell-${stimulus_row}-${stimulus_col}`);
            cell.style.backgroundColor = "";
          }
          stimulus_hidden = true;
          this.jsPsych.pluginAPI.setTimeout(() => {
            handleNoResponse();
          }, trial.isi_duration);
        }, trial.stimulus_duration);
      }
    };
    const handleResponse = (buttonIndex) => {
      if (!response_allowed || response_given) return;
      response_allowed = false;
      response_given = true;
      button_pressed = buttonIndex;
      const response_time = performance.now() - trial_start_time;
      let is_correct;
      if (!show_stimulus) {
        is_correct = buttonIndex !== trial.match_index;
      } else {
        is_correct = buttonIndex === trial.match_index && trial.is_target || buttonIndex !== trial.match_index && !trial.is_target;
      }
      this.jsPsych.pluginAPI.clearAllTimeouts();
      showFeedback(is_correct, response_time, true);
    };
    const handleNoResponse = () => {
      if (!response_allowed || response_given) return;
      response_allowed = false;
      response_given = true;
      const is_correct = show_stimulus ? !trial.is_target : true;
      showFeedback(is_correct, null, false);
    };
    const showFeedback = (is_correct, response_time, made_response) => {
      const elapsed_time = performance.now() - trial_start_time;
      let remaining_time;
      if (trial.stimulus_duration === null) {
        const isi_time = trial.isi_duration === null || trial.isi_duration === 0 ? 0 : trial.isi_duration;
        remaining_time = isi_time + trial.feedback_duration;
      } else {
        const total_trial_time = trial.stimulus_duration + trial.isi_duration + trial.feedback_duration;
        remaining_time = Math.max(0, total_trial_time - elapsed_time);
      }
      if (made_response || trial.feedback_duration > 0) {
        trial.buttons.forEach((buttonText, index) => {
          const button = document.getElementById(`nback-response-btn-${index}`);
          if (button) {
            button.disabled = true;
          }
        });
      }
      if (!trial.show_feedback_text && !trial.show_feedback_border) {
        if (show_stimulus && !stimulus_hidden) {
          if (trial.stimulus_duration === null) {
            const cell = document.getElementById(`cell-${stimulus_row}-${stimulus_col}`);
            cell.style.backgroundColor = "";
          } else {
            const stimulus_end_time = trial.stimulus_duration - elapsed_time;
            if (stimulus_end_time > 0) {
              this.jsPsych.pluginAPI.setTimeout(() => {
                const cell = document.getElementById(`cell-${stimulus_row}-${stimulus_col}`);
                cell.style.backgroundColor = "";
              }, stimulus_end_time);
            }
          }
        }
        this.jsPsych.pluginAPI.setTimeout(() => {
          endTrial(is_correct, response_time);
        }, remaining_time);
        return;
      }
      if (trial.feedback_duration === 0 && !made_response) {
        if (show_stimulus && !stimulus_hidden) {
          if (trial.stimulus_duration === null) {
            const cell = document.getElementById(`cell-${stimulus_row}-${stimulus_col}`);
            cell.style.backgroundColor = "";
          } else {
            const stimulus_end_time = trial.stimulus_duration - elapsed_time;
            if (stimulus_end_time > 0) {
              this.jsPsych.pluginAPI.setTimeout(() => {
                const cell = document.getElementById(`cell-${stimulus_row}-${stimulus_col}`);
                cell.style.backgroundColor = "";
              }, stimulus_end_time);
            }
          }
        }
        this.jsPsych.pluginAPI.setTimeout(() => {
          endTrial(is_correct, response_time);
        }, remaining_time);
        return;
      }
      const grid = document.getElementById("nback-grid");
      const feedback_div = document.getElementById("nback-feedback");
      const stimulus_cell = show_stimulus ? document.getElementById(`cell-${stimulus_row}-${stimulus_col}`) : null;
      if (trial.stimulus_duration === null && made_response) {
        const isi_time = trial.isi_duration === null || trial.isi_duration === 0 ? 0 : trial.isi_duration;
        if (stimulus_cell) {
          stimulus_cell.style.backgroundColor = "";
        }
        this.jsPsych.pluginAPI.setTimeout(() => {
          if (trial.show_feedback_border) {
            grid.style.border = `3px solid ${is_correct ? trial.correct_color : trial.incorrect_color}`;
          }
          if (trial.show_feedback_text) {
            let feedback_text = is_correct ? "Correct!" : "Incorrect!";
            if (response_time !== null) {
              feedback_text += ` (${Math.round(response_time)}ms)`;
            }
            feedback_div.textContent = feedback_text;
            feedback_div.style.color = is_correct ? trial.correct_color : trial.incorrect_color;
            feedback_div.style.visibility = "visible";
          }
        }, isi_time);
      } else {
        if (trial.show_feedback_border) {
          grid.style.border = `3px solid ${is_correct ? trial.correct_color : trial.incorrect_color}`;
        }
        if (trial.show_feedback_text) {
          let feedback_text = is_correct ? "Correct!" : "Incorrect!";
          if (response_time !== null) {
            feedback_text += ` (${Math.round(response_time)}ms)`;
          }
          feedback_div.textContent = feedback_text;
          feedback_div.style.color = is_correct ? trial.correct_color : trial.incorrect_color;
          feedback_div.style.visibility = "visible";
        }
      }
      if (show_stimulus && !stimulus_hidden) {
        if (trial.stimulus_duration === null) {
          if (made_response && stimulus_cell) {
            stimulus_cell.style.backgroundColor = "";
          }
        } else {
          const stimulus_end_time = trial.stimulus_duration - elapsed_time;
          if (stimulus_end_time > 0) {
            this.jsPsych.pluginAPI.setTimeout(() => {
              if (stimulus_cell) {
                stimulus_cell.style.backgroundColor = "";
              }
            }, stimulus_end_time);
          }
        }
      }
      this.jsPsych.pluginAPI.setTimeout(() => {
        endTrial(is_correct, response_time);
      }, remaining_time);
    };
    const endTrial = (is_correct, response_time, made_response) => {
      this.jsPsych.pluginAPI.clearAllTimeouts();
      const trial_data = {
        stimulus_row,
        stimulus_col,
        is_target: trial.is_target,
        response: button_pressed,
        response_time,
        correct: is_correct
      };
      this.jsPsych.finishTrial(trial_data);
    };
    createDisplay();
    startTrial();
  }
}

export { SpatialNbackPlugin as default };
//# sourceMappingURL=index.js.map
